import constants

class Modifier:

    def __init__(self, name):
        self.name = name

##     def __repr__(self):
##         return str(self)

##     def __str__(self):
##         return self.name

    def alignMod(self):
        """An object with this modifier's alignment is multiplied by
        this.  For objects like buildings with no inherent alignment,
        this substitutes for it"""
        return 1

    def attractMod(self):
        """An object with this modifier's attraction is multiplied by
        this"""
        return 1

    def upkeepMod(self):
        """Upkeep multiplier"""
        return 1

    def incomeMod(self):
        """Income multiplier.  This is always in respect to the
        player.  For example, adventurers who are big spenders will
        have a >1 multiplier here, because though it's their expenses,
        it's the player's income."""
        return 1

    def levelMod(self):
        """This is a multiplier for what level this character will
        appear to be"""
        return 1

class Famous(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Famous')

    def attractMod(self):
        return 1 + constants.AVG

class FreeLoot(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Free Loot')

    def attractMod(self):
        return 1 + constants.AVG

    def upkeepMod(self):
        return 1 + constants.HI

class FreeBed(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Free Bed')

# People classes

class Guard(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Guard')

    def alignMod(self):
        return 1 + constants.LOW

class PlotPoint(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Plot Point')

    def attractMod(self):
        return 1 + constants.HI

class Shady(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Shady')

    def alignMod(self):
        return -(1 + constants.LOW)

    def incomeMod(self):
        return 0.95

class SideQuest(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Side Quest')

    def attractMod(self):
        return 1 + constants.LOW / 2

    def levelMod(self):
        return 1.5

class Skilled(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Skilled')

    def incomeMod(self):
        return 1.1

    def attractMod(self):
        return 1.1

class Unskilled(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Unskilled')

    def incomeMod(self):
        return 0.9

    def attractMod(self):
        return 0.9

# Adventurer classes
class Hometown(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Hometown')

    def attractMod(self):
        return 2

class Rich(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Rich')

    def incomeMod(self):
        return 2

class Amateur(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Amateur')

    def levelMod(self):
        return 0.75

class AdventurerShady(Modifier):
    def __init__(self):
        Modifier.__init__(self, 'Shady')


# INSTANCES FOLLOW

# Building mods

famous = Famous()
freeLoot = FreeLoot()
freeBed = FreeBed()

buildingMods = [
    (famous,   2500),
    (freeLoot,  500),
    (freeBed,   100),
    ]

# Person mods
guard = Guard()
plotPoint = PlotPoint()
shady = Shady()
sideQuest = SideQuest()
skilled = Skilled()
unskilled = Unskilled()

personMods = [
    (guard,      500),
    (plotPoint, 2500),
    (shady,      500),
    (sideQuest, 1500),
    (skilled,   1000),
    (unskilled, 1000),
    ]

# Adventurer mods
hometown = Hometown()
rich = Rich()
amateur = Amateur()
adventurerShady = AdventurerShady()

adventureMods = [
    hometown, rich, amateur, shady,
    ]
