#!/usr/bin/env python

import widgets
import support
import gui
import town

import os
import pickle

class StartupState(gui.GUIState):
    def __init__(self, driver):
        gui.GUIState.__init__(self, driver)

        background = widgets.BackgroundColor( (0, 0, 128) )

        self.add(background)

        # ask for name of town
        text = "What would you like to name your town?"
        self.substate = support.NameAsker(self.driver(), self, text)
        self.townName = None
        self.mayorName = None

    def reactivate(self):
        if self.townName == None:
            self.townName = self.substate.input.text
            if not self.townName:
                self.driver().done()
            else:
                text = "What would you like to name your mayor?"
                self.substate = support.NameAsker(self.driver(), self,
                                                  text)
        elif self.mayorName == None:
            self.mayorName = self.substate.input.text
            if not self.mayorName:
                self.driver().done()
            else:
                strTown = str(self.townName)
                mayor = str(self.mayorName)
                theTown = town.TownScreen(self.driver(),
                                          strTown,
                                          mayor)

                self.driver().start(theTown)
        else:
            self.driver().done()

    def update(self):
        if self.townName == None or self.mayorName == None:
            self.driver().start(self.substate)                           

class LoadGameState(gui.GUIState):
    def __init__(self, driver):
        gui.GUIState.__init__(self, driver)

        background = widgets.BackgroundColor( (0, 0, 128) )

        self.add(background)

        text = "Load which game?"

        home = os.getenv("HOME")
        if not home:
            home = os.getcwd()
        dir = os.path.join(home, '.qct')

        self.files = []
        if os.access(dir, os.F_OK):
            files = os.listdir(dir)
            for file in files:
                self.files.append(os.path.join(dir, file))

        menu = []
        if self.files:
            for file in files:
                menu.append(file)
        else:
            text="There are no saved games to load!"
            menu.append('Nevermind')
        
        self.substate = support.MenuState(self.driver(), self, menu,
                                          text)
        self.result = None

    def reactivate(self):
        if self.files:
            filename = self.files[self.substate.chosen]
            theTown = pickle.load(open(filename))
            theTown._driver = self.driver()
            self.driver().replace(theTown)
        else:
            self.driver().done()

    def update(self):
        if self.result == None:
            self.driver().start(self.substate)
