#!/usr/bin/env python

import pygame.constants
import random
import os
import pickle

import buildings
import widgets
import entities
import gui
import support
import constants
import specializations
import modifiers
import turn

from gui import GUIState
from buildings import Tile

class TownScreen(GUIState):
    def __init__(self, driver, townName, mayorName):
        GUIState.__init__(self, driver)

        tilesx = 11
        tilesy = 13
        tilesw = 32
        tilesh = 32

        self.subState = None
        self.townspeople = []
        self.adventurers = []
        self.outsiders = [] # Townspeople not in town
        self.potentials = [] # Adventurers not in town
        self.villains = []
        self.villain = None
        self.money = 1000
        self.turn = 1
        self.reputation = 0.5
        self.villainFree = 0
        self.townName = townName

        self.font = constants.FONT
        font = self.font

        background = widgets.BackgroundColor( (0, 0, 128) )

        # Bottom menu
        self.menu = widgets.MenuBox( (10, 480 -
                                      font.size('OptionsEndTurn')[1] -
                                      14), 'arrow.png')
        self.menu.align = widgets.MenuBox.HORIZONTAL
        self.menu.xSpacing = 34
        self.menu.xStretch = 0
        self.menu.resize( (620, font.size('OptionsEndTurn')[1] + 4) )
        self.menu.add(widgets.Label(font, 'Options'))
        self.menu.add(widgets.Label(font, 'End Turn'))

        # Status counter
        status = 'Turn %d   $%d' % (self.turn, self.money)
        self.status = widgets.Label(font, status)

        # Tiles
        self.tiles = buildings.TileMenu( (10,15), 'cursor.png',
                                         (tilesx * tilesw, tilesy *
                                          tilesh))

        for x in range(0,tilesx):
            for y in range(0, tilesy):
                tile = Tile('grass.png', (x,y))
                self.tiles.add(tile)

        mayor = buildings.MayorHouse( (tilesx/2, 1) )
        mayor.modifiers.append(modifiers.freeBed)
        self.tiles.add(mayor)

        # Townspeople menu
        self.townMenu = widgets.MenuBox( (self.tiles.getAspect()[0] +
                                          self.tiles.getAspect()[2] +
                                          4,
                                          self.tiles.getAspect()[1]),
                                         'arrow.png')

        self.townMenu.xStretch = 0
        self.townMenu.allowScrolling = 1
        self.townMenu.resize( (130,self.tiles.getAspect()[3]) )

        mayorPerson = entities.Mayor(mayorName)
        self.addTownsperson(mayorPerson)
        mayor.moveIn(mayorPerson)

        #Adventurers menu
        self.adventMenu = widgets.MenuBox(
            (self.townMenu.getAspect()[0] +
             self.townMenu.getAspect()[2] + 4,
             self.tiles.getAspect()[1]), 'arrow.png')

        self.adventMenu.xStretch = 0
        self.adventMenu.yStretch = 0
        self.adventMenu.allowScrolling = 1
        self.adventMenu.resize( (130, self.tiles.getAspect()[3]) )

        adventurer = entities.Adventurer('adventurer.png')
        adventurer.setJob(specializations.warrior)
        self.addAdventurer(adventurer)

        # Focuser
        self.focus = TownFocuser(self.tiles, self.townMenu,
                                 self.adventMenu, self.menu)

        # add the various objects
        self.add(background)
        self.add(self.status)
        self.add(self.menu)
        self.add(self.tiles)
        self.add(self.townMenu)
        self.add(self.adventMenu)

        self.populateOutsiders()
        self.populatePotentials()
        self.populateVillains()

    def __getstate__(self):
        odict = self.__dict__.copy()
        del odict['font']
        del odict['subState']
        del odict['_driver']
        return odict

    def __setstate__(self, dict):
        self.__dict__.update(dict)
        self.font = constants.FONT
        self.subState = None

    def addAdventurer(self, entity):
        if not (entity in self.adventurers):
            self.adventurers.append(entity)
            self.adventMenu.add(entity)
        else:
            print "Tried to add dupe!", entity
        try:
            self.potentials.remove(entity)
        except ValueError:
            pass

    def removeAdventurer(self, entity):
        self.adventurers.remove(entity)
        self.adventMenu.remove(entity)
        self.potentials.append(entity)

    def addTownsperson(self, entity):
        if not (entity in self.townspeople):
            self.townspeople.append(entity)
            self.townMenu.add(entity)

    def avgLevel(self):
        levelTotal = 0.0
        levelCount = 0.0
        for tile in self.tiles.tileGen():
            if (isinstance(tile, buildings.Building) and
                (tile.job != specializations.building and
                 tile.job != specializations.construction)):
                levelCount += 1
                levelTotal += tile.job.effectiveLevel(tile)
        for person in self.townspeople:
            levelCount += 1
            levelTotal += person.job.effectiveLevel(person)
        return levelTotal / levelCount

    def removeTownsperson(self, entity):
        residence = entity.residence
        if residence:
            entity.residence.resident = None
        entity.residence = None
        self.townspeople.remove(entity)
        self.outsiders.append(entity)
        self.townMenu.remove(entity)
        self.townMenu.move(self.townMenu.location())

    def killRandomPerson(self):
        victims = filter(specializations.isNotMayor, self.townspeople)
        if len(victims) > 0:
            victim = random.choice(victims)
            self.removeTownsperson(victim)
            self.outsiders.remove(victim)
            return victim
        return None

    def killRandomBuilding(self):
        targets = []
        for building in self.tiles.getBuildings():
            if building.job != specializations.mayorHouse:
                targets.append(building)

        if len(targets) > 0:
            target = random.choice(targets)
            self.tiles.replace(target)
            return target
        return None
    
    def callback(self, object):
        if object == self.menu:
            self.menuAction(object.getChosenIndex())
        elif object == self.tiles:
            self.subState = buildings.TileScreen(self.driver(), self,
                                                 object.selectedTile())
            self.driver().start(self.subState)
        elif object == self.townMenu:
            self.subState = entities.PersonScreen(self.driver(), self,
                                                  object.getChosen())
            self.driver().start(self.subState)
            
    def endTurn(self):
        self.subState = turn.EndOfTurnScreen(self.driver(), self)
        self.driver().start(self.subState)

    def getTown(self):
        return self
        
##         # update construction
##         for tile in self.tiles.tileGen():
##             if isinstance(tile, buildings.Construction):
##                 tile.levelUp()
##                 if tile.level == 8:
##                     # construction complete
##                     newBuild = buildings.Building( ('house.png',), "House",
##                                          specializations.Building(),
##                                          tile.tileLoc)
##                     self.tiles.replace(tile, newBuild)

    def menuAction(self, index):
        if index == 0:
            choices = ( 'Save Game' , 'Return to Title Screen',
                        'Nevermind')
            self.subState = support.MenuState(self.driver(), self,
                                              choices)
            self.driver().start(self.subState)
        if index == 1:
            self.endTurn()

    def populateOutsiders(self):
        for x in range(constants.NUMOUTSIDERS):
            person = entities.randomTownsperson()
            person.level = random.randrange(1,50)
            person.picky = random.normalvariate(0.25, 0.1)
            person.align = random.normalvariate(0.5, 0.25)

            # how many mods will this character have?
            mods = int(random.normalvariate(0,1))
            for x in range(mods):
                chosen = random.choice(modifiers.personMods)[0]
                if not (chosen in person.modifiers):
                    person.modifiers.append(chosen)
            self.outsiders.append(person)

    def populatePotentials(self):
        for x in range(constants.NUMPOTENTIALS):
            spec, image = random.choice(specializations.adventSpecs)
            person = entities.Adventurer(image)
            person.setJob(spec)
            person.level = random.randrange(1,50)
            person.picky = random.normalvariate(0.25, 0.1)
            person.align = random.random()
            # how many mods will this character have?
            mods = int(random.normalvariate(0,1))
            for x in range(mods):
                chosen = random.choice(modifiers.adventureMods)
                if not (chosen in person.modifiers):
                    person.modifiers.append(chosen)

            self.potentials.append(person)

    def populateVillains(self):
        for x in range(constants.NUMVILLAINS):
            villain = entities.Villain()
            villain.level = random.randrange(25, 50)

            self.villains.append(villain)

    def reactivate(self):
        self.adventMenu.move(self.adventMenu.location())
        if self.subState:
            oldState = self.subState
            self.subState = None
            if isinstance(oldState, support.MenuState):
                # The bottom menu!
                if oldState.chosen == 0:
                    self.saveGame()
                elif oldState.chosen == 1:
                    self.driver().done()
            elif isinstance(oldState, turn.EndOfTurnScreen):
                # Came back from end of turn, move on
                self.turn += 1
                if self.villain == None:
                    self.villainFree += 1
                else:
                    self.villainFree = 0
            status = 'Turn %d   $%d' % (self.turn, self.money)
            self.status.setText(status)

    def saveGame(self):
        home = os.getenv("HOME")
        if not home:
            home = os.getcwd()
        dir = os.path.join(home, '.qct')
        filename = os.path.join(dir, self.townName)
        if not os.access(dir, os.F_OK):
            os.mkdir(dir)
        pickle.dump(self, open(filename, 'w'))
        
            
    def setVillain(self, villain):
        if villain:
            self.villains.remove(villain)
        else:
            if self.villain:
                self.villains.append(self.villain)

        self.villain = villain
            

class TownFocuser(gui.Focuser):
    def __init__(self, main, town, advent, bottom):
        gui.Focuser.__init__(self)
        self.add(main)
        self.add(town)
        self.add(advent)
        self.add(bottom)
        self.main = main
        self.advent = advent
        self.town = town
        self.bottom = bottom
        self.bottom.enable()

    def focusEvent(self, initiator):
        direction = initiator.direction
        if direction == pygame.constants.K_DOWN:
            self.focus(self.bottom)
        elif (direction == pygame.constants.K_UP and initiator ==
              self.bottom):
            self.focus(self.main)
        elif (direction == pygame.constants.K_LEFT and initiator ==
              self.advent):
            self.focus(self.town)
        elif (direction == pygame.constants.K_LEFT and initiator ==
              self.town):
            self.focus(self.main)
        elif (direction == pygame.constants.K_RIGHT and initiator ==
              self.main):
            self.focus(self.town)
        elif (direction == pygame.constants.K_RIGHT and initiator ==
              self.town):
            if self.advent:
                self.focus(self.advent)
