#!/usr/bin/env python

import gui
import widgets
import constants
import buildings
import specializations
import modifiers
import support

import random

class EndOfTurnScreen(gui.TransparentGUIState):
    """Displays the end of turn screen"""
    def __init__(self, driver, background):
        gui.TransparentGUIState.__init__(self, driver, background)

        container = widgets.BorderedContainer([60,60])
        self.font = constants.FONT
        self.substate = None
        focus = []

        self.updateConstruction()
        self.costs, self.income = self.updateEconomy()
        self.updatePotentials()

        self.outgoing = self.outgoingCheck()
        self.potentials = self.newcomersCheck()
        check = self.villainCheck()
        if check:
            status, villain = check
            if status == -1:
                villainText = ('Your adventurers successfully repelled the'+
                               ' villain!')
                self.background.setVillain(None)
            elif status == 0:
                villainText = ('The villain remains to continue the reign '+
                               'of terror!')
            elif status == 1:
                villainText = ('A villain has invaded and given your '+
                               'Adventurers a sound thrashing!')
                self.background.setVillain(villain)
            elif status == 2:
                villainText = ('Bored with your town, the villain is '+
                               'moving to greener, less charred' +
                               'pastures')
                self.background.setVillain(None)

            self.villain = villain
        else:
            villain = None
            self.villain = None

        eot = '- End of Turn %d -' % background.turn
        container.add(widgets.Label(self.font, eot))
        container.add(widgets.Label(self.font, ' ')) # space

        self.exitMenu = widgets.MenuBox( (100,100), 'arrow.png')
        self.exitMenu.align = widgets.MenuBox.HORIZONTAL
        self.exitMenu.add(widgets.Label(self.font, 'Done'))

        self.incomingMenu = widgets.MenuBox( (100,100), 'arrow.png')
        self.incomingMenu.align = widgets.MenuBox.HORIZONTAL
        self.incomingMenu.xSpacing = 20
        self.incomingMenu.xStretch = 0
        self.incomingMenu.allowScrolling = 1
        self.incomingMenu.resize( ( (container.size()[0] - 10),
                          40))
        for person in self.potentials:
            self.incomingMenu.add(person)
            self.background.addAdventurer(person)

        self.outgoingMenu = widgets.MenuBox( (100,100), 'arrow.png')
        self.outgoingMenu.align = widgets.MenuBox.HORIZONTAL
        self.outgoingMenu.xSpacing = 20
        self.outgoingMenu.xStretch = 0
        self.outgoingMenu.allowScrolling = 1
        self.outgoingMenu.resize( ( (container.size()[0] - 10),
                          40))

        self.villainMenu = widgets.MenuBox( (100,100), 'arrow.png')
        self.villainMenu.align = widgets.MenuBox.HORIZONTAL
        self.villainMenu.xSpacing = 20
        self.villainMenu.xStretch = 0
        self.villainMenu.allowScrolling = 1
        self.villainMenu.resize( ( (container.size()[0] - 10),
                          40))        
        for person in self.outgoing:
            self.outgoingMenu.add(person)
            self.background.removeAdventurer(person)

        focus.append(self.exitMenu)
        container.add(self.exitMenu)
        
        if len(self.potentials) > 0:
            incoming = ('The following adventurers came to your town '+
                        'today')
            self.incomingText = widgets.Label(self.font, incoming)
            container.add(self.incomingText)
            container.add(self.incomingMenu)
            focus.append(self.incomingMenu)
        else:
            incoming = ("No adventurers arrived today.")
            self.incomingText = widgets.Label(self.font, incoming)
            container.add(self.incomingText)

        if len(self.outgoing) > 0:
            outgoing = ('The following adventuers left your town '+
                        'today:')
            self.outgoingText = widgets.Label(self.font, outgoing)
            container.add(self.outgoingText)
            container.add(self.outgoingMenu)
            focus.append(self.outgoingMenu)
        else:
            incoming = ("No adventurers left today.")
            self.incomingText = widgets.Label(self.font, incoming)
            container.add(self.incomingText)

        if villain:
            self.villainMenu.add(villain)
            tmpWidget = widgets.Label(self.font, villainText)
            container.add(tmpWidget)
            container.add(self.villainMenu)
            focus.append(self.villainMenu)

        totalCost = 0
        totalIncome = 0
        for item in self.costs:
            totalCost += item[1]
        for item in self.income:
            totalIncome += item[1]

        if (villain and status != -1 and status != 2):
            totalIncome = 0
            text = self.villainAction(villain)
            container.add(widgets.Label(self.font, text))

        text = "Income ($%d) - " % totalIncome
        text = text + "Costs ($%d) = " % totalCost
        text = text + "Total Profit/loss: $%d" % (totalIncome -
                                                     totalCost)
        self.background.money += totalIncome - totalCost

        self.economy = widgets.Label(self.font, text)
        container.add(self.economy)

        self.container = container
        self.add(container)

        self.focus = gui.VerticalFocuser( focus )

    def callback(self, object):
        if object == self.exitMenu:
            self.done()

    def done(self):
        if self.background.turn % 7 == 0:
            next = EndOfWeekScreen(self.driver(), self.background)
            self.driver().replace(next)
        else:
            self.driver().done()

    def newcomersCheck(self, list = None, include = 1):
        background = self.background
        if list == None:
            potentials = background.potentials
        else:
            potentials = list

        results = []
        for person in potentials:
            cur = background.reputation
            alignDiff = 0.0
            levelTally = 0.0
            alignTotal = 0.0
            levelTotal = 0.0
            bonusMods = 1.0
            bonus = 1.0
            # average alignments and calculate levelTally
            for tile in background.tiles.tileGen():
                if (isinstance(tile, buildings.Building) and
                    tile.job != specializations.building and
                    tile.job != specializations.construction):
                    if tile.job.effectiveAlignment(tile) != None:
                        align = tile.job.effectiveAlignment(tile)
                        alignDiff += abs(align -
                                         person.job.effectiveAlignment(person))
                        alignTotal += 1
                    levelDiff = abs(tile.job.effectiveLevel(tile) -
                                    person.job.effectiveLevel(person))
                    if levelDiff < 5:
                        levelTally += ( 1.0 / (float(levelDiff)+1.0) )
                    else:
                        levelTally -= 0.2
                    levelTotal += 1

                    bonus += tile.job.comingToEffect(person, tile)
                    for item in tile.modifiers:
                        bonusMods *= item.attractMod()
                    
            for townie in background.townspeople:
                alignDiff += abs(townie.job.effectiveAlignment(townie) -
                                 person.job.effectiveAlignment(person))
                alignTotal += 1
                levelDiff = abs(townie.job.effectiveLevel(townie) -
                                person.job.effectiveLevel(person))
                
                if levelDiff < 5:
                    levelTally += ( 1.0 / (float(levelDiff)+1.0) )
                else:
                    levelTally -= 0.2
                levelTotal += 1
                bonus += townie.job.comingToEffect(person, townie)
                for item in townie.modifiers:
                    bonusMods *= item.attractMod()

            alignAvg = 1 - (alignDiff / alignTotal)

            # apply level bonus/penalty
            levelBonus = 1 + (levelTally - levelTotal/2) / levelTotal
            alignAvg *= levelBonus

            # apply starting reputation
            alignAvg *= cur + 0.5

            # apply bonuses
            alignAvg *= bonus * bonusMods

            if (1-alignAvg <= person.picky and
                person.away > 3 and
                person.distance < 20):
##                print (str(person) + " is in! avg %f dist %f" %
##                       (alignAvg, person.distance))
                if include:
                    results.append(person)
            else:
                if (1 - alignAvg <= person.picky):
                    person.distance -= 10 * person.wanderlust
                else:
                    person.distance += 5 * person.wanderlust
##                print (str(person) + " is out! avg %f dist %f picky %f" %
##                       (alignAvg, person.distance, person.picky))

                if not include:
                    results.append(person)

        return results

    def outgoingCheck(self):
        background = self.background

        results = []
        staying = 0
        room = 0
        # calculate how much room the town has
        for tile in background.tiles.tileGen():
            if isinstance(tile, buildings.Building):
                if tile.job == specializations.inn:
                    room += tile.level
                if modifiers.freeBed in tile.modifiers:
                    room += 1

        for person in background.adventurers:
            person.away = 0
            person.here += 1
            staying += 1
            if (person.here > person.wanderlust * 10 or
                staying > room):
                results.append(person)
                person.here = 0

        return results

    def updateConstruction(self):
        for tile in self.background.tiles.tileGen():
            if isinstance(tile, buildings.Construction):
                tile.levelUp()
                if tile.level == 8:
                    newBuild = buildings.Building( ('house.png',), "House",
                                                   specializations.building,
                                                   tile.tileLoc)
                    self.background.tiles.replace(tile, newBuild)

    def updateEconomy(self):
        costs = []
        income = []

        for tile in self.background.tiles.tileGen():
            if isinstance(tile, buildings.Building):
                costs.append( (tile.job.name, tile.job.upkeep(tile)) )
                for person in self.background.adventurers:
                    income.append( (tile.job.name,
                                    tile.job.income(person, tile)))

        for townie in self.background.townspeople:
            costs.append( (townie.name + ' (%s)' % townie.job.name,
                           townie.job.upkeep(townie)))
            for person in self.background.adventurers:
                income.append( (townie.name + ' (%s)' %
                                (townie.job.name),
                                townie.job.income(person, townie)))

        return costs, income

    def updatePotentials(self):
        for potential in self.background.potentials:
            direction = random.choice( [-1.0, 1.0] )
            amount = random.randrange(
                int(potential.wanderlust * 10.0) +1 )
            potential.distance += amount * direction
            potential.away += 1
            if potential.distance < 0: potential.distance = 0
            if potential.distance > constants.MAXDIST:
                potential.distance = constants.MAXDIST

    def villainAction(self, villain):
        evil = random.random()
        alignBias = 0.5
        if villain.job == specializations.destroyer:
            evilCap = .5
            personChance = .5
        elif villain.job == specializations.evil:
            evilCap = .25
            personChance = .1
            alignBias = 1
        elif villain.job == specializations.benign:
            evilCap = 0
            personChance = 0
        elif villain.job == specializations.conqueror:
            evilCap = .2
            personChance = .1
            alignBias = 0

        if evil < evilCap:
            # there's evil to be done!
            person = random.random()
            if person < personChance:
                # person has to die
                victim = self.background.killRandomPerson()
                if victim:
                    return "The villain has slain %s!" % victim.name
            else:
                # building gets knocked down
                building = self.background.killRandomBuilding()
                if building:
                    return ("The villain has destroyed the %s!" %
                            building.job.name)
                
        return ("The villain is busy watching everyone scurry about.")

    def villainCheck(self, villainComing = None):
        if self.background.villain or villainComing:
            if villainComing:
                villain = villainComing
            else:
                villain = self.background.villain
            villain.away = 0
            levelTotal = 0
            # Stack him up against the adventurers in town
            for advent in self.background.adventurers:
                levelTotal += advent.level
            levelBonus = random.random() + 0.5
            levelTotal *= levelBonus
            if levelTotal > villain.level * constants.VILLAINMULT:
                # we win!
                return (-1, villain)
            else:
                # No deal
                villain.here += 1
                if villain.here > 7:
                    return (2, villain)
                if villainComing:
                    return (1, villain)
                else:
                    return (0, villain)
        elif self.background.villainFree > constants.VILLAINFREE:
            minDiff = 9999
            found = None
            for villain in self.background.villains:
                villain.away += 1
                villain.here = 0
                diff = abs(villain.level - self.background.avgLevel())
                if diff < minDiff:
                    found = villain
                    minDiff = diff
            diff = found.level - self.background.avgLevel()
            if (diff >= 0 and
                diff <= 10 and
                found.away > constants.VILLAINFREE * 2):
                chance = diff / 10.0
                chosen = random.random()
                if chosen < chance:
                    return self.villainCheck(found)
                else:
                    pass
        return None

class EndOfWeekScreen(gui.TransparentGUIState):
    """The end of week screen"""
    def __init__(self, driver, background):
        gui.TransparentGUIState.__init__(self, driver, background)

        container = widgets.BorderedContainer([60,60])
        self.font = constants.FONT
        self.substate = None
        focus = []
        self.done = 0

        eow = '- End of Week %d -' % (background.turn / 7)
        container.add(widgets.Label(self.font, eow))
        container.add(widgets.Label(self.font, ' '))
        
        self.exitMenu = widgets.MenuBox( (100,100), 'arrow.png')
        self.exitMenu.align = widgets.MenuBox.HORIZONTAL
        self.exitMenu.add(widgets.Label(self.font, 'Done'))

        self.potentials = self.newcomersCheck()
        self.incomingMenu = widgets.MenuBox( (100,100), 'arrow.png')
        self.incomingMenu.align = widgets.MenuBox.HORIZONTAL
        self.incomingMenu.xSpacing = 20
        self.incomingMenu.xStretch = 0
        self.incomingMenu.allowScrolling = 1
        self.incomingMenu.resize( ( (container.size()[0] - 10),
                          40))
        for person in self.potentials:
            self.incomingMenu.add(person)

        focus.append(self.exitMenu)
        container.add(self.exitMenu)
        if len(self.potentials) > 0:
            incoming = 'The following people wish to join your town:'
            self.incomingText = widgets.Label(self.font, incoming)
            container.add(self.incomingText)
            container.add(self.incomingMenu)
            focus.append(self.incomingMenu)
        else:
            incoming = ('There are no people wishing to join your '+
                        'town.')
            self.incomingText = widgets.Label(self.font, incoming)
            container.add(self.incomingText)

##            container.add(widgets.Label(self.font, incoming))
            
        self.container = container
        self.add(container)

        self.focus = gui.VerticalFocuser( focus )

    def callback(self, object):
        if object == self.exitMenu:
            if not self.done:
                self.driver().done()
            self.done = 1
        elif object == self.incomingMenu:
            person = self.incomingMenu.getChosen()
            caption = 'Move %s into town?' % person.name
            self.substate = support.Confirm(self.driver(), self,
                                            caption)
            self.driver().start(self.substate)

    def newcomersCheck(self):
        background = self.background
        outsiders = background.outsiders
        population = float(len(background.townspeople))
        results = []
        for person in outsiders:
            cur = background.reputation
            alignDiff = 0.0
            levelPeople = 0.0
            # average alignments and calculate 'levelPeople'
            for townPerson in background.townspeople:
                if not (modifiers.shady in townPerson.modifiers and
                        modifiers.shady in person.modifiers):
                    alignDiff += abs(townPerson.align - person.align)

                levelDiff = abs(townPerson.level - person.level)
                if levelDiff == 0:
                    levelPeople += 1
                elif levelDiff == 1:
                    levelPeople += 0.5
                elif levelDiff == 2:
                    levelPeople += 0.25
                elif levelDiff == 3:
                    levelPeople += 0.1
                elif levelDiff >= 4:
                    levelPeople -= 0.2
            
            alignAvg = alignDiff / population
            alignAvg = 1 - alignAvg
            # Check for open houses and apply bonus
            holding = 0
            for tile in background.tiles.tileGen():
                if isinstance(tile, buildings.Building):
                    holding += tile.canHold()
            popDiff = holding - population
            if popDiff <= 0:
                bonus = 0.0
            else:
                try:
                    bonus = 1.0 + (holding - popDiff) / float(holding)
                except ZeroDivisionError:
                    bonus = 0.0
            alignAvg *= bonus

            # Apply level bonus/penalty
            levelBonus = 1 + (levelPeople - population/2) / population
            alignAvg *= levelBonus
            
            # Apply starting reputation
            alignAvg *= cur + 0.5
                        
            if 1-alignAvg <= person.picky:
                results.append(person)
            else:
                pass
                
        return results

    def reactivate(self):
        if self.substate.result == 0:
            # move 'em in!
            person = self.incomingMenu.getChosen()
            self.background.outsiders.remove(person)
            self.background.addTownsperson(person)
            self.incomingMenu.clear()
            self.potentials = self.newcomersCheck()
            if len(self.potentials) > 0:
                for person in self.potentials:
                    self.incomingMenu.add(person)
            else:
                self.focus.remove(self.incomingMenu)
                self.container.remove(self.incomingMenu)
                incoming = ('There are no more people wishing to '+
                            'join your town.')
                self.incomingText.setText(incoming)
                
